#include "a_station.h"
#include <stdlib.h>
#include <string.h>

// Создаёт пустую станцию
struct station* create_station() {
    struct station* station = (struct station*)calloc(1, sizeof(struct station));
    station->tracks = (struct car**)calloc(STATION_SIZE, sizeof(struct car*));
    station->track_count = STATION_SIZE;
    return station;
}

// Освобождает всю память
void destroy_station(struct station* station) {
    for (int i = 0; i < station->track_count; i++) {
        struct car* current = station->tracks[i];
        while (current) {
            struct car* to_free = current;
            current = current->next;
            free(to_free);
        }
    }
    free(station->tracks);
    free(station);
}

// Хеш-функция для выбора пути на основе названия конечной станции
int select_track(struct station* station, const char* target) {
    int hash = 0;
    for (int i = 0; target[i] != '\0'; i++) {
        hash = (hash + target[i]) % station->track_count;
    }
    return hash;
}

// Добавляет количество пассажиров для указанной станции
void add_target_capacity(struct station* station, const char* target, int capacity) {
    int index = select_track(station, target);
    struct car* current = station->tracks[index];
    
    while (current) {
        if (strcmp(current->value, target) == 0) {
            current->capacity += capacity;
            return;
        }
        current = current->next;
    }
    
    struct car* new_car = (struct car*)malloc(sizeof(struct car));
    strncpy(new_car->value, target, TARGET_SIZE - 1);
    new_car->value[TARGET_SIZE - 1] = '\0';
    new_car->capacity = capacity;
    new_car->next = station->tracks[index];
    station->tracks[index] = new_car;
}

// Получает количество пассажиров для указанной станции
int get_target_capacity(struct station* station, const char* target) {
    int index = select_track(station, target);
    struct car* current = station->tracks[index];
    
    while (current) {
        if (strcmp(current->value, target) == 0) {
            return current->capacity;
        }
        current = current->next;
    }
    return 0;
}

// Считает количество конечных станций
int count_targets(struct station* station) {
    int count = 0;
    for (int i = 0; i < station->track_count; i++) {
        struct car* current = station->tracks[i];
        while (current) {
            count++;
            current = current->next;
        }
    }
    return count;
}

// Считает общую вместимость всех записей
int count_capacity(struct station* station) {
    int total_capacity = 0;
    for (int i = 0; i < station->track_count; i++) {
        struct car* current = station->tracks[i];
        while (current) {
            total_capacity += current->capacity;
            current = current->next;
        }
    }
    return total_capacity;
}
